package CPAN::Test;

use strict;
use vars qw($VERSION @ISA);

$VERSION = '0.13';

require Class::Data::Inheritable;
@ISA = qw(Class::Data::Inheritable);

use CLASS;
use CPAN::Test::Module;
use CPAN::Test::Author;

my(%Authors, %Modules, %Modules_By_Namespace);


=head1 NAME

CPAN::Test - Utility functions for smoke testing CPAN.

=head1 SYNOPSIS

  use CPAN::Test;

  CPAN::Test->mirror($default_CPAN_mirror);
  CPAN::Test->init;

  my($authors) = CPAN::Test->init_authors($mailrc_file);
  my($modules) = CPAN::Test->init_packages($packages_file);

  my(@modules) = CPAN::Test->read_update_email($update_email);
  my $author = CPAN::Test->author($id);
  my $module = CPAN::Test->module_by_archive($archive);
  my $module = CPAN::Test->module_by_namespace($namespace);

=head1 DESCRIPTION

A collection of utility methods and functions for smoking CPAN
modules.  It mostly contains methods for reading CPAN index files and
generating CPAN::Test::Module and CPAN::Smoke::Author objects.

=head2 Initialization

These should be called before doing anything else.

=for testing
use CPAN::Test;

=over 4

=item B<mirror>

  my $CPAN_mirror = CPAN::Test->mirror;
  CPAN::Test->mirror($CPAN_mirror);

Sets the default CPAN mirror modules will use to download themselves.
If this isn't set, you have to set each module individually.

=for testing
can_ok( 'CPAN::Test', 'mirror' );

=cut

CLASS->mk_classdata('mirror');


=item B<init>

  CPAN::Test->init;

Downloads the indexes from the given mirror() and runs the init_*
methods.

This is currently implemented very badly.

=for testing
can_ok( 'CPAN::Test', 'init' );

=cut

use LWP::Simple qw(getstore);
use Archive::Any;

sub init {
    my($class) = shift;
    my $mirror = $class->mirror;

    getstore join('/', $mirror, 'modules', '02packages.details.txt.gz'),
             '.02packages.details.txt.gz';
    getstore join('/', $mirror, 'authors', '01mailrc.txt.gz'),
             '.01mailrc.txt.gz';

    `gunzip -f .02packages.details.txt.gz`;
    `gunzip -f .01mailrc.txt.gz`;

    CPAN::Test->init_authors('.01mailrc.txt');
    CPAN::Test->init_packages('.02packages.details.txt');

    unlink '.01mailrc.txt', '.02packages.details.txt.gz';
}


=item B<init_authors>

  my $authors = CPAN::Test->init_authors($mailrc_file);

Initializes the list of authors based on the authors/01mailrc.txt CPAN
index file.

=for testing
ok( CPAN::Test->can('init_authors'),   'init_authors' );

=cut

sub init_authors {
    my($class, $mailrc) = @_;

    open(AUTHORS, $mailrc) or die "Can't open $mailrc: $!";
    while( <AUTHORS> ) {
        my($id, $name, $email) = m/^alias \s+ 
                                    (\S+) \s+ 
                                    "([^\"\<]+) \s+ <(.+)>"
                                  /x;

        $Authors{$id} = CPAN::Test::Author->create({ id    => $id,
                                                      name  => $name,
                                                      email => $email,
                                                    });
    }

    return \%Authors;
}

=item B<init_packages>

  my $packages = init_packages($package_file);

Initializes the list of CPAN modules based on the
modules/02packages.details.txt.gz file.  You should call
init_authors() first.

=for testing
ok( CPAN::Test->can('init_packages'),  'init_packages' );

=cut

sub init_packages {
    my($class, $package_file) = @_;
    my %info = ();

    open(PACK, $package_file) or die "Can't open $package_file:  $!";

    # Read the headers.
    # Mail-header  KEY: VALUE style with blank line ending.
    while(<PACK>) {
        last if /^\s*$/;

        my($key, $val) = m/^(.+?):\s*(.*?)$/;
        $info{$key} = $val;
    }


    warn "Couldn't find a line count.\n" 
      unless defined $info{'Line-Count'};

    my $default_mirror = CPAN::Test->mirror;
    my $lines = 0;
    while(<PACK>) {
        chomp;
        $lines++;
        my($mod, $version, $file_path, $comment) = split " ", $_, 4;
        $version = undef if $version eq 'undef';

        my($authorid, $file) = $file_path =~ m{^.+?/.+?/(.+?)/(.+)$}; #}

        my $module;
        unless( $module = $Modules{$file} ) {
            my $author = CPAN::Test->author($authorid);
            $module = CPAN::Test::Module->create({ 
                             author     => $author,
                             file       => $file,
                             filepath   => $file_path,
                            });

            $module->which_mirror($default_mirror) if $default_mirror;
            $author->add_modules($module) if defined $author;

            $Modules{$file} = $module;
        }                             

        $module->add_namespaces($mod => $version);
        $Modules_By_Namespace{$mod} = $module;
    }

    warn "Wrong line count!\n" unless $lines == $info{'Line-Count'};

    return \%Modules;
}


=back


=head2 Getting modules and authors.

=over 4

=item B<author>

  my $author = CPAN::Test->author($author_id);

Returns a CPAN::Test::Author from the list produced by running
init_authors().

=for testing
can_ok( 'CPAN::Test', 'author' );

=cut

sub author {
    return $Authors{$_[1]};
}

=item B<module_by_archive>

  my $module = CPAN::Test->module_by_archive($archive);

Returns a CPAN::Test::Module for the given $archvie (such as
'Sex-0.69.tar.gz') from the list produced by running init_packages().

=for testing
can_ok( 'CPAN::Test', 'module_by_archive' );

=cut

sub module_by_archive {
    return $Modules{$_[1]};
}

=item B<module_by_namespace>

    my $module = CPAN::Test->module_by_namespace($namespace);

Returns a CPAN::Test::Module for the given $namespace (such as
'Sex').

=for testing
can_ok( 'CPAN::Test', 'module_by_namespace' );

=cut

sub module_by_namespace {
    return $Modules_By_Namespace{$_[1]};
}

=item B<read_update_email>

    my @modules = CPAN::Test->read_update_email($file);

Reads the email sent out by CPAN updates and returns a list of the
modules updated as CPAN::Test::Module objects.

=for testing
can_ok( 'CPAN::Test', 'read_update_email' );

=cut

use Mail::Internet;

sub read_update_email {
    my($class, $email) = @_;

    open(FILE, $email) or do { warn "Can't open $email:  $!";  return };
    my @email = <FILE>;

    my $body = Mail::Internet->new(\@email)->body;

    until( $body->[0] =~ /^\[\d+\.\]/ ) {
        shift @$body;
    }

    my @modules = ();
    foreach my $line (@$body) {
        if( $line =~ /^\s*CPAN id:\s+(\S+)/ ) {
            my $module = CPAN::Test::Module->new;
            push @modules, $module;

            my($id) = $1 =~ m|/([^/]+)$|;
            $module->author(CPAN::Test->author($id));
        }
        elsif( $line =~ /^\s*Package:\s+(\S+)/ ) {
            if( my $module = CPAN::Test->module_by_archive($1) ) {
                $modules[-1] = $module;
            }
            else {
                $modules[-1]->file($1);
            }
        }
    }

    return @modules;
}


=back

=cut

1;
