package CPAN::Test::Mistake;

use strict;
use vars qw($VERSION @ISA);

$VERSION = 0.13;

use CLASS;
require Class::Accessor;
@ISA = qw(Class::Accessor);

use Data::Denter;
$Data::Denter::HashMode = 1;

my %Mistakes = Undent(join '', <DATA>);


=head1 NAME

CPAN::Test::Mistake - Objects representing fuck-ups in CPAN modules

=head1 SYNOPSIS

  use CPAN::Test::Mistake;

  my $mistake = CPAN::Test::Mistake->new($mistake_name, $more_info);

  my @mistakes = CPAN::Test::Mistake->all;

  my $by    = $mistake->by;
  my $fatal = $mistake->is_fatal;
  my $name  = $mistake->name;
  my $desc  = $mistake->description;
  my $fix   = $mistake->fix;
  my $info  = $mistake->more_info;

  my $report = $mistake->report;

=head1 DESCRIPTION

An object representing a mistake made when smoking a CPAN module.  It
could be a mistake by the author, by CPAN or by CPAN::Test itself.

Mistakes are read in from a config file.

=for testing
use CPAN::Test::Mistake;
$CLASS = 'CPAN::Test::Mistake';

=head2 Constructors

=over 4

=item B<new>

  my $mistake = CPAN::Test::Mistake->new($name, $more_info);

Generates a new mistake.  $name must be in the list of known mistakes.
$more_info is further information about the mistake 

=for testing
can_ok( $CLASS, 'new' );

=cut

sub new {
    my($class, $name, $more_info) = @_;

    my $mistake = $Mistakes{$name};
    return unless defined $mistake;

    $mistake->{name} = $name;
    $mistake->{more_info} = $more_info if defined $more_info;
    my $self = $class->SUPER::new($mistake);

    return $self;
}

=item B<all>

  my @mistakes = CPAN::Test::Mistake->all;

Simply returns a list of all available Mistakes in no particular order.

=begin testing
can_ok( $CLASS, 'all' );

my @mistakes = CPAN::Test::Mistake->all;
ok( @mistakes,      'all() returned some mistakes' );
ok( !grep !$_->isa('CPAN::Test::Mistake'), @mistakes );

=end testing

=cut

sub all {
    my($class) = shift;
    return map $class->new($_), keys %Mistakes;
}

=back

=head2 Output

=over 4

=item B<report>

  my $report = $mistake->report;

Collects together all the information in this mistake into a human
friendly format suitable for sending to a module author.

=cut

sub report {
    my($self) = shift;

    my %by_map = (
                  Author    => 'you',
                  Internal  => 'us',
                  CPAN      => 'CPAN'
                 );


    my $report = <<REPORT;
We may have found a %s mistake in your module.  It appears to have
been made by %s.

Mistake:  %s
%s

Our Recommendation:
%s
REPORT

    return sprintf $report, $self->is_fatal ? 'fatal' : 'minor',
                            $by_map{$self->by},
                            $self->name,
                            $self->description,
                            $self->fix;
}

=back

=head2 Accessors

Simple get/set accessor methods.

=over 4

=item B<by>

Who made the mistake?  

  CPAN      => CPAN itself is wrong
  Internal  => A mistake by CPAN::Test
  Author    => The module author made the goof.

=cut

=for testing
can_ok( $CLASS, 'by' );

=cut

CLASS->mk_accessors('by');

=item B<is_fatal>

Is the mistake fatal, or can we recover from it?  True or false.

=for testing
can_ok( $CLASS, 'is_fatal' );

=cut

CLASS->mk_accessors('is_fatal');

=item B<name>

Cannonical name of this mistake.

=for testing
can_ok( $CLASS, 'name' );

=cut

CLASS->mk_accessors('name');

=item B<description>

A detailed description of the problem.

=for testing
can_ok( $CLASS, 'description');

=cut

CLASS->mk_accessors('description');

=item B<fix>

A suggestion of how to fix it like, 'Contact modules@perl.org'

=for testing
can_ok( $CLASS, 'fix' );

=cut

CLASS->mk_accessors('fix');

=item B<more_info>

Additional diagnostic information about this particular instance of
the mistake.

=for testing
can_ok( $CLASS, 'more_info' );

=cut

CLASS->mk_accessors('more_info');

=back

=cut

=head1 AUTHOR

Michael G Schwern E<lt>schwern@pobox.comE<gt>

=head1 SEE ALSO

CPAN::Test

=cut


1;


__DATA__
File Not Found => %
    by => CPAN
    is_fatal => 1
    description => Couldn't find your module on CPAN
    fix => Contact modules@perl.org

Naughty Archive => %
    by => Author
    is_fatal => 1
    descripton => Your archive is trying to do NAUGHTY THINGS!  See Archive::Any for an exact definition of naughty.
    fix => Use make dist instead of rolling your own archvies.

Impolite Archive => %
    by => Author
    is_fatal => 0
    description => Your archive didn't make it's own directory to unpack into.
    fix => Use make dist instead of rolling your own archives.

Bad Archive => %
    by => Author
    is_fatal => 1
    description => Your archive appears to be corrupt.
    fix => Use make dist instead of rolling your own archives.

Fetch Dir Doesn't Exist => %
    by => Internal
    is_fatal => 1
    description => Can't find the directory I'm supposed to download modules into
    fix => Make a directory, dumbass

Can't Fetch Module => %
    by => Internal
    is_fatal => 1
    description => Can't download the module from CPAN
    fix => Contact modules@perl.org maybe?

Couldn't Run Makefile.PL => %
    by => Internal
    is_fatal => 1
    description => There's a Makefile.PL, but I can't run it at all.

No Makefile.PL => %
    by => Author
    is_fatal => 1
    description => Your module has no Makefile.PL.
    fix => Write a Makefile.PL.  Read perlmodtut.

Unknown Mistake => %
    by => Internal
    is_fatal => 1
    description => Tried to declare a mistake of an unknown type.
    fix => Have a look at the source code.

Failed Tests => %
    by => Author
    is_fatal => 1
    description => The module failed a 'make test'.
    fix => Examine the test output and fix your code!

Failed make => %
    by => Author
    is_fatal => 1
    description => The module failed a 'make'.
    fix => Examine the make output and fix your code!

Tests in test.pl => %
    by => Author
    is_fatal => 0
    description => Module's tests are in test.pl instead of t/*.t and will not be parsed by 'make test'
    fix => Move your tests into t/foo.t instead of test.pl.  Make sure they have the right output for a real test.  See Test::Tutorial.

No tests => %
    by => Author
    is_fatal => 0
    description => The module has no tests at all!
    fix => Write some tests.  See Test::Tutorial

Failed Install => %
    by => Author
    is_fatal => 1
    description => The module failed a 'make install'.
    fix => Examine the install output and fix it.

No Makefile => %
    by => Author
    is_fatal => 1
    description => There was no Makefile after running 'perl Makefile.PL'
    fix => Examine the Makefile.PL output and fix it

Old Perl => %
    by => Internal
    is_fatal => 1
    description => We tried to install your module with a Perl older than you allow.  Our bad.
