package CPAN::Test::Module;

use strict;
use vars qw($VERSION @ISA);

$VERSION = 0.13;

use Archive::Any;
use Carp::Assert;
use CLASS;
use File::chdir;
use Fatal qw(chdir mkdir);
use File::Spec::Functions qw(:DEFAULT rel2abs splitdir);

require Class::Accessor;
@ISA = qw(Class::Accessor);

CLASS->mk_accessors(qw(_fetched_to _mistakes _unpacked_to));


=head1 NAME

CPAN::Test::Module - representations of CPAN modules

=head1 SYNOPSIS

  use CPAN::Test::Module;

  my $module = CPAN::Test::Module->create(\%module_data);
  $module->which_mirror($cpan_mirror);
  $module->fetch($to_some_dir);
  $module->unpack($to_some_other_dir);
  $module->which_perl($perl);
  $module->prefix($dir);
  $module->makefile_pl(@args);
  my %prereqs = $module->prereqs;
  $module->make;
  $module->make_other($target);
  $module->test;
  $module->install;
  $module->clean;       #UNIMPLEMENTED#

  my %namespaces = $module->namespaces;
  $module->add_namespaces($namespace => $version);

  my @mistakes = $module->mistakes;
  $module->mistake($mistake_name, $more_info);

  # Simple accessors
  $module->name;
  $module->author;
  $module->file_path;
  $module->file;
  $module->mkpl_output;
  $module->make_output;
  $module->test_output;

  $module->reset;

=head1 DESCRIPTION

A representation of a CPAN module for the purposes of smoking it.

A CPAN module is that which is encased in a single archive (tarball or
zip file) and is typically made up of one or more .pm files with tests
spanning one more or packages.

These objects can do useful things like download, unpack and build
themselves.  These are B<very raw> operations.  They're not going to
help you (or get in your way) by, for instance, making sure you've
unpacked a module before testing it.  If you want that, write
wrappers.

=for testing
require_ok('CPAN::Test::Module');
$CLASS = 'CPAN::Test::Module';

=head2 Constructors

=over 4

=item B<create>

  my $module = CPAN::Test::Module->create(\%module_data);

Creates a new module object with the given data.

=for testing
can_ok( 'CPAN::Test::Module', 'create' );

=cut

sub create {
    my($class, $data) = @_;

    my $self = $class->SUPER::new($data);

    $self->which_perl($^X) unless $self->which_perl;
    $self->_namespaces({});
    $self->_mistakes([]);

    return $self;
}

=back

=head2 Information about the module.

=over 4

=item B<name>

  $module->name($name);
  my $name = $module->name;

The cannonical name of this module (ie. 'Test::Harness').  If not
supplied, it will take a guess based on the filename and namespaces.
It will always return something.

=for testing
can_ok( $CLASS, 'name' );

=cut

CLASS->mk_accessors('_name');

sub name {
    my($self) = shift;

    if( @_ ) {
        $self->_name(shift);
    }

    unless( defined $self->_name ) {
        $self->_name($self->_guess_name_from_namespaces);
    }

    return $self->_name;
}


sub _guess_name_from_file {
    my($self) = shift;
    my $file = $self->file;
    my($name, $version) = $file =~ m/^([A-Za-z_0-9-]+)-([\d\._]+)\./;
    $name =~ s/-/::/g;

    return($name, $version);
}

sub _guess_name_from_namespaces {
    my($self) = shift;

    my %ns = $self->namespaces;
    my($cand_name, $cand_version) = $self->_guess_name_from_file;

    return $cand_name if exists $ns{$cand_name} && 
                         $ns{$cand_name} == $cand_version;

    while( my($ns, $ver) = each %ns ) {
        if( defined $ver && $ver == $cand_version ) {
            return $ns;
        }
    }

    # Last ditch.
    return $self->file;
}


=item B<author>

  $module->author($author);
  my $author = $module->author;

Author of the module, as CPAN::Test::Author object.

=for testing
can_ok( 'CPAN::Test::Module', 'author' );

=cut

CLASS->mk_accessors('author');

=item B<file>

  $module->file($file);
  my $file = $module->file;

The file this module comes in (like 'Sex-0.69.tar.gz').

=for testing
can_ok( 'CPAN::Test::Module', 'file' );

=cut

CLASS->mk_accessors('file');

=item B<filepath>

  my $filepath = $module->filepath;

The full CPAN file path to this module (like
'M/MS/MSCHWERN/Sex-0.69.tar.gz')

=for testing
can_ok( 'CPAN::Test::Module', 'filepath' );

=cut

CLASS->mk_accessors('_filepath');

sub filepath {
    my($self) = shift;

    if( @_ ) {
        $self->_filepath(@_);
    }
    elsif( !$self->_filepath ) {
        $self->_filepath($self->author->dir . '/' . $self->file);
    }

    return $self->_filepath;
}
   

=item B<namespaces>

  my %namespaces = $module->namespaces;

The namespaces contained in this module and their versions.

    namespace => version

=for testing
can_ok( 'CPAN::Test::Module', 'namespaces' );

=cut

CLASS->mk_accessors('_namespaces');

sub namespaces {
    my($self) = shift;

    return %{$self->_namespaces};
}   

=item B<add_namespaces>

  $module->add_namespaces($namespace => $version,
                          $another_namespace => $its_version,
                          ...
                         );

Add namespaces, with their versions, to this module.

=for testing
can_ok( 'CPAN::Test::Module', 'add_namespaces' );

=cut

sub add_namespaces {
    my($self, %spaces) = @_;
    my $namespaces = $self->_namespaces;

    while( my($new_space, $version) = each %spaces ) {
        warn "$new_space already exists" if exists $namespaces->{$new_space};
     
        $namespaces->{$new_space} = $version;
    }

    return 1;
}

=item B<mkpl_output>

  my $mkpl_output = $module->mkpl_output;

Result of the last makefile_pl() run.

=item B<make_output>

  my $make_output = $module->make_output;

Result of the last make() run.

=item B<test_output>

  my $test_output = $module->test_output;

Result of the last test() run.

=item B<install_output>

  my $install_output = $module->install_output;

Result of the last install() run.

=for testing
can_ok( $CLASS, qw(mkpl_output make_output test_output install_output) );

=cut

CLASS->mk_accessors(qw(mkpl_output make_output test_output install_output));

=item B<reset>

  $module->reset;

Resets the state of the module for a new testing run.

Currently only clears out the various *_output methods.

=cut

sub reset {
    my($self) = @_;
    $self->mkpl_output('');
    $self->make_output('');
    $self->test_output('');
    $self->install_output('');
}

=back


=head2 Unpacking and building

=over 4

=item B<which_mirror>

  $module->which_mirror($cpan_mirror);

What CPAN mirror should we fetch this module from?  This is a full URL
like 'http://www.cpan.org/CPAN'.

You have to set this before you fetch().

=for testing
can_ok( 'CPAN::Test::Module', 'which_mirror' );

=cut

CLASS->mk_accessors('which_mirror');

=item B<fetch>

  my $filepath = $module->fetch($dir);

Goes out and grabs a copy of this module from CPAN (using the entry
from $module->which_mirror) and puts it in the given directory.  If no 
directory is given, it will use the Cwd.

Returns the $filepath to the download.

=for testing
can_ok( 'CPAN::Test::Module', 'fetch' );

=cut

# LWP::Simple's get() confuses Class::Accessor's get().
use LWP::Simple qw(getstore is_success);

sub fetch {
    my($self, $dir) = @_;

    $dir = curdir() unless defined $dir;

    $self->mistake("Fetch Dir Doesn't Exist", $dir) unless -x $dir;

    my $url = join '/', $self->which_mirror, 'authors/id', $self->filepath;

    my $file = catfile($dir, $self->file);
    unless( is_success(getstore($url, $file)) ) {
        $self->mistake("Can't Fetch Module", "$url -> $file");
    }

    $self->_fetched_to($dir);

    return $file;
}

=item B<unpack>

  my $unpacked_dir = $module->unpack($dir);

Unpacks the given module into the given directory.  If no directory is
given, it will unpack it in the same directory it was fetched into.

Returns the directory the module unpacked to.

=for testing
can_ok( 'CPAN::Test::Module', 'unpack' );

=cut

use Archive::Any;

sub unpack {
    my($self, $dir) = @_;

    my $archive = Archive::Any->new(catfile($self->_fetched_to, $self->file));

    my $new_dir;
    $dir = $self->_fetched_to unless defined $dir;
    local $CWD = $dir;

    if( $archive->is_naughty ) {
        $self->mistake('Naughty Archive', $self->file);
        return;
    }

    if( $archive->is_impolite ) {
        $self->mistake('Impolite Archive', $self->file);

        my $tmp_dir = $self->author->id . $$ . time;
        mkdir $tmp_dir;
        local $CWD = $tmp_dir;
        $archive->extract or $self->mistake("Bad Archive", $self->file);
        $new_dir = catdir($dir, $tmp_dir);
    }
    else {
        my($tmp_dir) = splitdir(($archive->files)[0]);
        $new_dir = catdir($dir, $tmp_dir);
        $archive->extract or $self->mistake("Bad Archive", $self->file);
    }

    $self->_unpacked_to($new_dir);

    return $new_dir;
}

=item B<which_perl>

  my $perl = $module->which_perl;
  $module->which_perl($perl);

Get/set which perl should be used to build & test this module.
If not set, it defaults to the currently running perl.

=for testing
can_ok( 'CPAN::Test::Module', 'which_perl' );

=cut

CLASS->mk_accessors('which_perl');

=item B<prefix>

  my $prefix = $module->prefix;
  $module->prefix($prefix);

Get/set which directory the module gets installed into (thing PREFIX).

This will be converted to an absolute path.

=for testing
can_ok( 'CPAN::Test::Module', 'prefix' );

=cut

CLASS->mk_accessors('_prefix');
sub prefix {
    my($self) = shift;

    if( @_ ) {
        $self->_prefix(rel2abs($_[0]));
    }
    return $self->_prefix;
}

=item B<makefile_pl>

  $module->makefile_pl(@args);

Runs the Makefile.PL against this module using $module->which_perl.

Any arguments are passed to the Makefile.PL on the command line.

=for testing
can_ok( 'CPAN::Test::Module', 'makefile_pl' );

=cut

sub makefile_pl {
    my($self, @args) = @_;

    local $CWD = $self->_unpacked_to;

    $self->mistake("No Makefile.PL") unless -r 'Makefile.PL';

    my $perl = $self->which_perl;
    if( my $prefix = $self->prefix ) {
        my $lib  = catdir($prefix, 'lib/');
        my $man1 = catdir($prefix, 'man/man1/');
        my $man3 = catdir($prefix, 'man/man3/');

        unshift @args, "PREFIX=$prefix";
        unshift @args, "LIB=$lib";
        unshift @args, "INSTALLMAN1DIR=$man1";
        unshift @args, "INSTALLMAN3DIR=$man3";
    }

    my $output = '';
    eval {
        local $SIG{ALRM} = sub { die "ALARM!\n"; };
        alarm 30;

        local $ENV{PERL_MM_USE_DEFAULT} = 1;
        my $pid = open(MPL, '-|') || 
          exec "echo /dev/null | $perl Makefile.PL @args 2>&1";
        $self->mkpl_output(join '', <MPL>);

        $self->mistake("Couldn't Run Makefile.PL", $!) unless $pid;

        alarm 0;
    };
    if( $@ ) {
        if( $@ eq "ALARM!\n" ) {
            $self->mistake("Makefile.PL timed out", $output);
        }
        elsif( ref $@ && $@->isa('CPAN::Mistake::Exception') ) {
            # Propogate CPAN::Mistake exceptions.
            die $@;
        }
        else {
            $self->mistake("UNKNOWN", $output);
        }
    }

    if( $self->mkpl_output =~ 
          /^Perl (\d+\.\d+) required--this is only version \d+\.\d+, stopped/m )
    {
        $self->mistake('Old Perl', $1);
    }

    $self->mistake("No Makefile") unless -f 'Makefile';

    return 1;
}

=item B<prereqs>

    my %prereqs = $module->prereqs;

Returns the declared prerequisites for this module based on the
PREREQ_PM in their Makefile.PL.  Must be run *after* makefile_pl().

=for testing
can_ok( $CLASS, 'prereqs' );

=cut

sub prereqs {
    my($self) = shift;

    local $CWD = $self->_unpacked_to;

    my $fh;
    open($fh, "Makefile") or die "Can't find the Makefile: $!";

    my %p = ();
    while (<$fh>) {
        last if /MakeMaker post_initialize section/;
        my($p) = m{^[\#]
                    \s+PREREQ_PM\s+=>\s+(.+)
                  }x;
        next unless $p;

        while ( $p =~ m/(?:\s)([\w\:]+)=>q\[(.*?)\],?/g ){
            # In case a prereq is mentioned twice, complain.
            if ( defined $p{$1} ) {
                warn "Warning: PREREQ_PM mentions $1 more than once, ".
                     "last mention wins";
            }
            $p{$1} = $2;
        }
        last;
    }

    return %p;
}


=item B<make>

    my $is_ok = $module->make;

Simply runs 'make' and returns pass or fail.

Make output is stored in $module->make_output;

=for testing
can_ok( $CLASS, 'make' );

=cut

sub make {
    my($self) = shift;

    local $CWD = $self->_unpacked_to;

    $self->make_output(scalar `make 2>&1`);

    $self->mistake('Failed make', $?) if $?;

    return !$?;
}


=item B<test>

    my $is_ok = $module->test;

Simply runs 'make test' and returns pass or fail.

Test output is stored in $module->test_output;

=for testing
can_ok($CLASS, 'test');

=cut

sub test {
    my($self) = shift;

    local $CWD = $self->_unpacked_to;

    opendir TEST_DIR, 't/';
    my @tests = grep /\.t$/, readdir TEST_DIR;
    close TEST_DIR;

    $self->mistake('Tests in test.pl') if -f 'test.pl' && !@tests;

    # This makes mistakes on SDKs.
#    $self->mistake('No tests') unless (-f "test.pl") || @tests;

    $self->test_output(scalar `make test 2>&1`);

    $self->mistake('Failed Tests', $?) if $?;

    return !$?;
}

=item B<install>

  $module->install;

Installs the module into the directory specified by $module->prefix.

=cut

sub install {
    my($self) = shift;

    local $CWD = $self->_unpacked_to;

    $self->install_output(scalar `make install 2>&1`);

    $self->mistake('Failed Install', $?) if $?;

    # XXX Need to add a check to see if things actually got installed.

    return !$?;
}

=back

=head2 Mistakes

When modules go bad!  These are methods for recording whe modules make
mistakes.

=over 4

=item B<mistake>

  my $mistake = $module->mistake($name, $info);

Records a mistake made by a module.  Any fatal mistake made will cause
an exception, the fatal CPAN::Test::Mistake object will be thrown.
Trying to make a non-existant mistake will also cause an exception.

Returns the new CPAN::Test::Mistake object.

=for testing
can_ok( $CLASS, 'mistake' );

=cut

CLASS->mk_accessors('_mistakes');

require CPAN::Test::Mistake;
sub mistake {
    my($self, $name, $info) = @_;

    my $mistake = CPAN::Test::Mistake->new($name, $info);

    my $mistakes = $self->_mistakes;
    push @$mistakes, $mistake;

    unless( defined $mistake ) {
        my($pack, $file, $line) = caller;
        die CPAN::Test::Mistake->new("Unknown Mistake", 
                                 "$name - $info at line $line of $file\n");
    }

    die $mistake if $mistake->is_fatal;

    return $mistake;
}


=item B<mistakes>

  my @mistakes = $module->mistakes;

Returns the mistakes made by this module in the order they were made.

=for testing
can_ok( $CLASS, 'mistakes' );

=cut

sub mistakes {
    my($self) = shift;

    return @{$self->_mistakes};
}

=back

=cut

=head1 AUTHOR

Michael G Schwern E<lt>schwern@pobox.comE<gt>

=head1 SEE ALSO

CPAN::Test

=cut


1;

