#!/usr/bin/perl -w

use strict;
use Test::More 'no_plan';
use File::Path;

use CPAN::Test::Module;
use CPAN::Test;

use Carp;
$SIG{__DIE__} = sub {
    if( ref $_[0] and $_[0]->isa('CPAN::Test::Mistake') ) {
        confess sprintf "Unexpected Mistake %s from %s at line %d\n%s",
                          $_[0]->name, (caller)[1,2], $_[0]->more_info;
        
    }
    else {
        die @_;
    }
};

BEGIN {
    mkdir 't/build';
    mkdir 't/lib';
    mkdir 't/man';
}

CPAN::Test->mirror('file:t/CPAN/');
CPAN::Test->init_authors('t/01mailrc.txt');
CPAN::Test->init_packages('t/02packages.details.txt');
my $schwern = CPAN::Test->author('MSCHWERN');
isa_ok( $schwern, 'CPAN::Test::Author' );

my $sex = CPAN::Test::Module->create({
                                  author   => $schwern,
                                  file     => 'Sex-0.69.tar.gz',
                                  filepath => 'M/MS/MSCHWERN/Sex-0.69.tar.gz'
                                 });

isa_ok( $sex, 'CPAN::Test::Module' );
is( $sex->author->id,  'MSCHWERN',                      '  author correct' );
is( $sex->filepath,   'M/MS/MSCHWERN/Sex-0.69.tar.gz',  '  filepath');
is( $sex->file,        'Sex-0.69.tar.gz',               '  file' );

ok( !$sex->namespaces,  'no namespaces yet' );

$sex->add_namespaces(Sex => 0.69);
ok( eq_hash( {$sex->namespaces}, { Sex => 0.69 } ),     'add_namespaces' );

is( $sex->name,       'Sex',                            '  name() guess' );

my $japhy = CPAN::Test->author('PINYAN');
isa_ok( $japhy, 'CPAN::Test::Author' );

my $yape = CPAN::Test::Module->create({
                                  author   => $japhy,
                                  file     => 'YAPE-Regex-3.01.tar.gz',
                                 });
isa_ok( $yape, 'CPAN::Test::Module' );
is( $yape->author->id,  'PINYAN',                       '  author' );
is( $yape->file,        'YAPE-Regex-3.01.tar.gz',       '  file' );
is( $yape->filepath,    'P/PI/PINYAN/YAPE-Regex-3.01.tar.gz',  
                                               '  filepath autogen' );

$yape->add_namespaces( YAPE::Regex          => 3.01,
                       YAPE::Regex::Element => 3.00,
                     );
ok( eq_hash( {$yape->namespaces}, { YAPE::Regex => 3.01,
                                    YAPE::Regex::Element => 3.00 } ),
                                     'add_namespaces multiple' );

is( $yape->name,        'YAPE::Regex',                  '  name() guess' );

is( $yape->which_perl,  $^X,    'which_perl() default' );
$yape->which_perl('/usr/local/bin/perl');
is( $yape->which_perl, '/usr/local/bin/perl',   'which_perl() set');

$yape->which_mirror('file:t/CPAN/');
is( $yape->which_mirror, 'file:t/CPAN/',        'which_mirror()' );

my $file = $yape->fetch('t');
is( $file, 't/YAPE-Regex-3.01.tar.gz',      'fetch() thinks it worked' );
ok( -e $file,                               '  and it actually did'    );

END { ok( unlink( $file ),  'YAPC fetch cleanup' ); }


my $new_dir = $yape->unpack('t/build');
is( $new_dir,   't/build/YAPE-Regex-3.01',  'unpack() thinks it worked' );
is( $yape->mistakes, 0,                     '  there were no mistakes' );
ok( -x $new_dir,                            "  yep, there's a directory" );
$yape->makefile_pl;
ok( -r 't/build/YAPE-Regex-3.01/Makefile',        '  made a Makefile' );
ok( eq_hash( { $yape->prereqs }, { Text::Balanced => 0 } ), '  prereq()' );



# your.pm is a nice, simple module with no prereqs.
my $module = CPAN::Test->module_by_namespace('your');
is( $module->which_mirror,      'file:t/CPAN/', 'your.pm has default mirror' );
$module->fetch('t');
$module->unpack('t/build');
ok( -d 't/build/your-0.01/',         '  unpack() worked' );
$module->prefix('t');
$module->makefile_pl;
ok( -r 't/build/your-0.01/Makefile', '  makefile_pl() produced a Makefile' );
ok( eq_hash( {$module->prereqs}, {} ),  '  has no prereqs' );
ok( $module->make,                      '  make()' );
ok( $module->test,                      '  tests()' );
like( $module->test_output, qr/^All tests successful/m,  '  test_output()' );
ok( $module->install,                   '  install()' );

END { ok( rmtree([<t/build/*>]),  't/build cleanup' ) }
END { ok( rmtree([<t/lib/*>]),    't/lib cleanup' ) }
END { ok( rmtree([<t/man/*>]),    't/man cleanup' ) }


# Tests for mistake() and mistakes()

my $mistake = $yape->mistake('Impolite Archive', $yape->file);
isa_ok( $mistake, 'CPAN::Test::Mistake' );
is( $mistake->name,         'Impolite Archive' );
is( $mistake->more_info,    $yape->file );

eval {
    local $SIG{__DIE__} = 'DEFAULT';
    $yape->mistake('Naughty Archive', $yape->file);
};
isa_ok( $@,  'CPAN::Test::Mistake' );
is( $@->name,       'Naughty Archive',      'mistake() has the right name' );
is( $@->more_info,  $yape->file,            '  and more_info' );

my @mistakes = $yape->mistakes;
is( @mistakes,  2,                              'mistakes()' );
is( $mistakes[0]->name,     'Impolite Archive', '  first one' );
is( $mistakes[1]->name,     'Naughty Archive',  '  second one'  );


my $mistake_line;
eval {
    local $SIG{__DIE__} = 'DEFAULT';
    $mistake_line = __LINE__ + 1;
    $yape->mistake("I don't exist you fool", "fooble");
};
isa_ok( $@, 'CPAN::Test::Mistake' );
is( $@->name,       'Unknown Mistake',  'Unknown mistake exception' );
is( $@->more_info,  
        "I don't exist you fool - fooble at line $mistake_line of $0\n" );
